package OPCUATankSystem "Package containing the model used in the example"
  model ExampleModel "Example using the OPC UA library"
    OPCUA.Examples.Utilities.SimpleTank tank1(h(start = 1)) annotation(Placement(visible = true, transformation(origin = {-40, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Blocks.Sources.Step step(offset = 0.25, height = 0.65, startTime = 5) annotation(Placement(visible = true, transformation(origin = {30, -70.318}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Blocks.Continuous.LimPID PI(k = 1, Ti = 0.005, yMax = 1, Ni = 0.1, initType = Modelica.Blocks.Types.Init.InitialOutput, controllerType = Modelica.Blocks.Types.SimpleController.PI, Td = 0.1) annotation(Placement(transformation(extent = {{-56, -20}, {-36, 0}}, rotation = 0, origin = {101, -0}), visible = true));
    OPCUA.Examples.Utilities.ControlledTank tank2(h(start = 1), maxFlow = 1) annotation(Placement(visible = true, transformation(origin = {75, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Blocks.Sources.Pulse pulse(amplitude = 0.45, period = 1, offset = 0, startTime = 7, nperiod = 1) annotation(Placement(visible = true, transformation(origin = {-80, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    OPCUA.Client client(errorHandling = OPCUA.Types.ErrorHandling.ErrorWhenUncertain) annotation(Placement(visible = true, transformation(origin = {55, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Clocked.ClockSignals.Clocks.PeriodicExactClock clock(factor = 200) annotation(Placement(visible = true, transformation(origin = {20, 26}, extent = {{-6, -6}, {6, 6}}, rotation = 0)));
    Modelica.Clocked.ClockSignals.Sampler.ShiftSample shift(shiftCounter = 1, resolution = 10) annotation(Placement(visible = true, transformation(origin = {56, 26}, extent = {{-6, -6}, {6, 6}}, rotation = 0)));
    OPCUA.Components.Unclocked.RealSignals.Write write(uaRealType = OPCUA.Types.UAReal.Double, nodeID = "51031", nodeIDType = OPCUA.Types.UANodeIdType.NumericId, nodeNamespace = 1, errorHandling = OPCUA.Types.ErrorHandling.ErrorWhenUncertain) annotation(Placement(visible = true, transformation(origin = {0, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    OPCUA.Components.Unclocked.RealSignals.Read read(nodeIDType = OPCUA.Types.UANodeIdType.NumericId, nodeID = "51031", nodeNamespace = 1, errorHandling = OPCUA.Types.ErrorHandling.ErrorWhenUncertain) annotation(Placement(visible = true, transformation(origin = {0, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  equation
    connect(step.y, tank2.qIn) annotation(Line(visible = true, origin = {56.75, -67.659}, points = {{-15.75, -2.659}, {4.25, -2.659}, {4.25, 2.659}, {7.25, 2.659}}, color = {0, 0, 127}));
    connect(tank2.y, PI.u_m) annotation(Line(visible = true, origin = {58, -57.333}, points = {{6, -17.667}, {-3, -17.667}, {-3, 35.333}}, color = {0, 0, 127}));
    connect(PI.y, tank2.u) annotation(Line(visible = true, origin = {82.5, -40.159}, points = {{-16.5, 30.159}, {6.5, 30.159}, {6.5, -24.841}, {3.5, -24.841}}, color = {0, 0, 127}));
    connect(pulse.y, tank1.qIn) annotation(Line(visible = true, origin = {-60.5, 60}, points = {{-8.5, 0}, {8.5, 0}}, color = {0, 0, 127}));
    connect(tank1.h, write.u) annotation(Line(visible = true, origin = {-20.5, 60}, points = {{-8.5, 0}, {8.5, 0}}, color = {0, 0, 127}));
    connect(write.list_a, client.writeList) annotation(Line(visible = true, origin = {27.5, 60}, points = {{-17.5, 0}, {17.5, 0}}, color = {0, 128, 0}, pattern = LinePattern.Dot, thickness = 0.5));
    connect(clock.y, client.writeClock) annotation(Line(visible = true, origin = {34.9, 40.5}, points = {{-8.3, -14.5}, {0.1, -14.5}, {0.1, 14.5}, {8.1, 14.5}}, color = {175, 175, 175}, pattern = LinePattern.Dot, thickness = 0.5));
    connect(clock.y, shift.u) annotation(Line(visible = true, origin = {37.7, 26}, points = {{-11.1, 0}, {11.1, 0}}, color = {175, 175, 175}, pattern = LinePattern.Dot, thickness = 0.5));
    connect(shift.y, client.readClock) annotation(Line(visible = true, origin = {69.9, 40.5}, points = {{-7.3, -14.5}, {5.1, -14.5}, {5.1, 14.5}, {-2.9, 14.5}}, color = {175, 175, 175}, pattern = LinePattern.Dot, thickness = 0.5));
    connect(read.y, PI.u_s) annotation(Line(visible = true, origin = {23.5, -23.5}, points = {{-12.5, -13.5}, {-3.5, -13.5}, {-3.5, 13.5}, {19.5, 13.5}}, color = {0, 0, 127}));
    connect(read.list_a, client.readList) annotation(Line(visible = true, origin = {30.829, 15}, points = {{-40.829, -45}, {-45.842, -45}, {-45.842, 0}, {49.171, 0}, {49.171, 45}, {34.171, 45}}, color = {0, 128, 0}, pattern = LinePattern.Dot, thickness = 0.5));
    annotation(Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Ellipse(visible = true, lineColor = {0, 114, 195}, fillColor = {0, 114, 195}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}), Polygon(visible = true, lineColor = {0, 114, 195}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-36, 60}, {64, 0}, {-36, -60}, {-36, 60}})}), preferredView = "info", experiment(StopTime = 20, __Wolfram_SynchronizeWithRealTime = false), uses(Modelica(version = "4.0.0")), Documentation(info = "<html><p>In this example, sampled data from one system is used to control the behavior of another, and the OPC server works as the communication link between these two systems.</p>
<p>This example requiers an OPC UA example server running on <code>localhost</code>. Read about how to start the example server <a href=\"modelica://OPCUA.Documentation.GettingStarted#start-server\">here</a>.</p>
<p>The first system consists of a simple tank (<code>tank1</code>) with an outlet. The tank is fed with a medium in a pulse-wise manner determined by component <code>pulse</code>. The height of the medium in the tank is sampled every 0.2 seconds and written to an OPC UA Server (using <code>client</code>). The sampled data represents a height profile that should be mimicked in <code>tank2</code> in the second system. To achieve this behavior, the height data read from the server by <code>client</code> is used as a reference signal to the PID component that controls the behavior of the second tank.</p>
<p><img style=\"display: block; margin-left: auto; margin-right: auto;\" src=\"modelica://OPCUATankSystem.ExampleModel/DocumentationFiles/TankLevel.png\" width=\"570\" height=\"815\"></p></html>", figures = {Figure(title = "Output and Reference", identifier = "a8fd5", preferred = true, plots = {Plot(identifier = "tank1", curves = {Curve(y = write.uSampled, legend = "Tank 1 Output"), Curve(y = tank1.h, legend = "Tank 1 Level")}), Plot(identifier = "tank2", curves = {Curve(y = read.y, legend = "Tank 2 PID Reference"), Curve(y = tank2.h, legend = "Tank 2 Level")})}, caption = "The level of tank1 and the sampled signal written to the OPC server is shown in %(plot:tank1).
The reference signal read from the OPC server) fed to the PID controller (and the level of tank2 and  is shown in %(plot:tank2)."), Figure(title = "Measured and Reference Tank Level", identifier = "5c5ed", plots = {Plot(curves = {Curve(y = PI.u_s, legend = "Tank Level Reference"), Curve(y = PI.u_m, legend = "Tank Level Measurement")})}), Figure(title = "Tank 1 Inflow and Outflow", identifier = "77bfb", plots = {Plot(curves = {Curve(y = tank1.qIn, legend = "Tank 1 Inflow"), Curve(y = tank1.qOut, legend = "Tank 1 Outflow")})}), Figure(title = "Tank 2 Inflow and Outflow", identifier = "1504d", plots = {Plot(curves = {Curve(y = tank2.qIn, legend = "Tank 2 Inflow"), Curve(y = tank2.qOut, legend = "Tank 2 Outflow")})}), Figure(title = "Tank Levels", identifier = "335db", plots = {Plot(curves = {Curve(y = tank1.h, legend = "Level Tank 1"), Curve(y = tank2.h, legend = "Level Tank 2")})})}), Diagram(coordinateSystem(extent = {{-115, -100}, {110, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics = {Rectangle(visible = true, origin = {-60, 59.841}, fillColor = {217, 223, 240}, fillPattern = FillPattern.Solid, extent = {{-40, -20.159}, {40, 20.159}}), Polygon(visible = true, origin = {35, -43.545}, fillColor = {217, 223, 240}, fillPattern = FillPattern.Solid, points = {{0, 53.545}, {60, 53.545}, {60, -46.773}, {-60, -46.773}, {-60, -6.773}, {0, -6.773}})}));
  end ExampleModel;
  annotation(preferredView = "info", uses(Modelica(version = "4.0.0"), OPCUA(version = "3.0")), experiment(__Wolfram_SynchronizeWithRealTime = false), __Wolfram(openClass = OPCUATankSystem.ExampleModel), Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Text(visible = true, origin = {0.0, 21.0}, lineColor = {255, 0, 0}, extent = {{-120.0, 73.0}, {120.0, 132.0}}, textString = "%name", fontName = "Arial"), Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90.0, -90.0}, {90.0, 90.0}}, radius = 25), Text(visible = true, origin = {12.0, 20.8333}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-102.0, -70.8333}, {78.0, 29.1667}}, textString = "Ex", fontName = "Arial")}));
end OPCUATankSystem;
